/**
 * @file WriteProfile.cpp
 * @date 2010/04/10 新規作成
 * @author
 *
 * INIファイルパラメータ取得クラス
 */
#include <stdio.h>
#include <fstream>
#include <sstream>
#include "Define.h"
#include "TrimSpace.h"
#include "GetProfile.h"
#include "ParseLine.h"
#include "WriteProfile.h"

/**
 * コンストラクタ
 */
WriteProfile::WriteProfile(std::string path){
	m_Filename = path;
}

/**
 * 該当セクション？
 * @param section セクション名
 * @return true/false
 */
bool WriteProfile::isPertinentSection(std::string section, std::string string){
	TrimSpace trim_space;
	string = trim_space.trimEnds(string);
	string.erase(0, 1);
	string.erase(string.length() - 1);
	if( string.compare(section.c_str()) == 0 ){
		return true;
	}
	return false;
}

/**
 * 該当name？
 * @param name 値名
 * @return true/false
 */
bool WriteProfile::isPertinentName(std::string name, std::string string){
	TrimSpace trim_space;
	std::string::size_type index = string.find('=', 0);
	string = trim_space.trimEnds(string.substr(0, index));
	if( string.compare(name.c_str()) == 0 ){
		return true;
	}
	return false;
}

/**
 * int値変更
 * @param section セクション名
 * @param name 値名
 * @param value値
 * @return 処理結果 OK/NG
 */
ReturnValue WriteProfile::writeInt(std::string section, std::string name, int value){
	std::string backup_filename = m_Filename + ".bak";
	if( rename(m_Filename.c_str(), backup_filename.c_str()) == 0 ){
		std::stringstream buffer, original;
		buffer << value;
		GetProfile	get_profile(backup_filename.c_str());
		original << get_profile.getInt(section, name);
		std::ifstream input(backup_filename.c_str());
		std::ofstream output(m_Filename.c_str());
		writeProfile(section, name, buffer.str(), original.str(), input, output);
		input.close();
		output.close();
		remove(backup_filename.c_str());
		return OK;
	}else{
		return NG;
	}
}

/**
 * string値変更
 * @param section セクション名
 * @param name 値名
 * @param value値
 * @return 処理結果 OK/NG
 */
ReturnValue WriteProfile::writeString(std::string section, std::string name, std::string value){
	std::string backup_filename = m_Filename + ".bak";
	if( rename(m_Filename.c_str(), backup_filename.c_str()) == 0 ){
		GetProfile	get_profile(backup_filename.c_str());
		std::ifstream input(backup_filename.c_str());
		std::ofstream output(m_Filename.c_str());
		writeProfile(section, name, value, get_profile.getString(section, name).c_str(), input, output);
		input.close();
		output.close();
		remove(backup_filename.c_str());
		return OK;
	}else{
		return NG;
	}
}

/**
 * ファイルに書き込み
 * @param section セクション名
 * @param name 値名
 * @param value値(std::string)
 * @return 処理結果 OK/NG
 */
#include <stdio.h>
ReturnValue WriteProfile::writeProfile(std::string section, std::string name, std::string value, std::string original_value,
										std::ifstream& input, std::ofstream& output){
	char buffer[256];
	std::string string_buffer;
	bool is_pertinent_section = false;
	ParseLine parse_line;
	IniLine previous_line = INVALID;

	if( input != false ){
		while( input.eof() != true ){
			input.getline(buffer, sizeof(buffer));
			if( previous_line == NULL_LINE ){
				string_buffer = '\n';
				output.write(string_buffer.c_str(), string_buffer.length());
				previous_line = INVALID;
			}
			switch(parse_line.parse(buffer)){
			case SECTION :
				if( is_pertinent_section == false ){
					is_pertinent_section = isPertinentSection(section, buffer);
				}
			case COMMENT :
				string_buffer = buffer;
				string_buffer += '\n';
				output.write(string_buffer.c_str(), string_buffer.length());
				break;
			case NULL_LINE :
				previous_line = NULL_LINE;
				break;
			case PARAMETER :
				if( (is_pertinent_section == true) && (isPertinentName(name, buffer) == true) ){
					string_buffer = buffer;
					if( original_value.length() > 0 ){
						//string_buffer.replace(string_buffer.find(name, 0), name.length(), name);
						string_buffer.replace(string_buffer.find(original_value, 0), original_value.length(), value);
						string_buffer += '\n';
					}else{
						string_buffer += value;
						string_buffer += '\n';
					}
					output.write(string_buffer.c_str(), string_buffer.length());
					is_pertinent_section = false;
				}else{
					string_buffer = buffer;
					string_buffer += '\n';
					output.write(string_buffer.c_str(), string_buffer.length());
				}
				break;
			case INVALID :
			default :
				break;
			}
		}
	}

	return OK;
}

